import { aws_iam, aws_lambda, Duration, StackProps } from 'aws-cdk-lib';
import { Construct } from 'constructs';
export interface LambdaProps {
    [key: string]: String;
}
/**
 * Manages Database Migration
 */
export interface Props extends StackProps {
    /**
     * Migration to apply, when the resource is created
     */
    readonly up: string;
    /**
     * Migration to apply, when the resource is deleted
     */
    readonly down: string;
    /**
     * One or more cdk constructs the migration depends on
     */
    readonly dependsOn?: Construct | Construct[];
    /**
     * Timeout of the lambda function, which is used to run the migrations
     *
     * @default - 30 seconds
     */
    readonly timeout?: Duration;
}
/**
 * Defines a new DB Migration
 */
export declare class Base extends Construct {
    /**
     * The lambda function which backs the custom resource
     */
    readonly lambda: aws_lambda.IFunction;
    /**
     * The IAM role, used by the lambda function which backs the custom resource
     */
    readonly role: aws_iam.Role;
    /**
     * Migration to apply, when the resource is deleted
     */
    readonly up: string;
    /**
     * Migration to apply, when the resource is deleted
     */
    readonly down: string;
    /**
     * Defines a new DB Migration
     */
    constructor(scope: Construct, id: string, props: Props);
    protected makeProperties(_: Props): LambdaProps;
    protected makeType(): string;
}
